@@ -1,7 +1,7 @@
 /*
     C-Dogs SDL
     A port of the legendary (and fun) action/arcade cdogs.
-    Copyright (c) 2013-2016, 2018 Cong Xu
+    Copyright (c) 2013-2016, 2018-2019 Cong Xu
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -53,6 +53,8 @@ CPicDrawContext CPicDrawContextNew(void)
 	c.Offset = svec2i_zero();
 	c.Radians = 0;
 	c.Scale = svec2_one();
+	c.Mask = colorWhite;
+	c.Flip = SDL_FLIP_NONE;
 	return c;
 }
 
@@ -287,6 +289,12 @@ void CPicDraw(
 	GraphicsDevice *g, const CPic *p,
 	const struct vec2i pos, const CPicDrawContext *context)
 {
+	CPicDrawContext ctx;
+	if (context == NULL)
+	{
+		ctx = CPicDrawContextNew();
+		context = &ctx;
+	}
 	const Pic *pic = CPicGetPic(p, context->Dir);
 	if (pic == NULL)
 	{
@@ -294,6 +302,7 @@ void CPicDraw(
 	}
 	const struct vec2i picPos = svec2i_add(pos, context->Offset);
 	PicRender(
-		pic, g->gameWindow.renderer, picPos, p->Mask, context->Radians,
-		context->Scale);
+		pic, g->gameWindow.renderer, picPos, ColorMult(p->Mask, context->Mask),
+		context->Radians,
+		context->Scale, context->Flip, Rect2iZero());
 }
