@@ -2,7 +2,7 @@
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
 
-	Copyright (c) 2017, 2019, 2021 Cong Xu
+	Copyright (c) 2017, 2019, 2021, 2023-2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -32,12 +32,11 @@
 #include "gauge.h"
 
 #define WAIT_MS 1000
+#define FLASH_PERIOD_MS 100
 
 void HealthGaugeInit(HealthGauge *h)
 {
-	h->health = 0;
-	h->waitHealth = 0;
-	h->waitMs = 0;
+	memset(h, 0, sizeof *h);
 }
 
 void HealthGaugeUpdate(HealthGauge *h, const TActor *a, const int ms)
@@ -79,8 +78,8 @@ void HealthGaugeDraw(
 	}
 
 	HSV hsv = {0.0, 1.0, 1.0};
-	const int health = actor->health;
-	const int maxHealth = ActorGetCharacter(actor)->maxHealth;
+	const int maxHealth = ActorGetMaxHeal(actor, false);
+	const int health = MIN(actor->health, maxHealth);
 	if (actor->poisoned)
 	{
 		hsv.h = 120.0;
@@ -93,6 +92,12 @@ void HealthGaugeDraw(
 		hsv.h =
 			(maxHealthHue - minHealthHue) * health / maxHealth + minHealthHue;
 	}
+	// Flash bar white if just took damage
+	if (h->waitMs > 0 && (h->waitMs % FLASH_PERIOD_MS) < FLASH_PERIOD_MS / 2)
+	{
+		hsv.s = 0.0;
+		hsv.v = 1.0;
+	}
 	color_t barColor;
 	if (h->health != health)
 	{
@@ -110,9 +115,25 @@ void HealthGaugeDraw(
 	barColor.a = opts.Mask.a;
 	HUDDrawGaugeInner(device, &gPicManager, pos, innerWidth, barColor);
 
+	// Draw a second bar if health is over max
+	if (actor->health > maxHealth)
+	{
+		const int healthOverMax = actor->health - maxHealth;
+		const int excessHealth = ActorGetMaxHeal(actor, true);
+		const int excessHealthRange =
+			MAX(excessHealth - maxHealth, healthOverMax);
+		const int innerWidth2 =
+			MAX(1, width * healthOverMax / excessHealthRange);
+		hsv.h = 120.0;
+		hsv.v = 1.0;
+		barColor = ColorTint(colorWhite, hsv);
+		barColor.a = opts.Mask.a;
+		HUDDrawGaugeInner(device, &gPicManager, pos, innerWidth2, barColor);
+	}
+
 	// Draw health number label
 	char s[50];
-	sprintf(s, "%d", health);
+	sprintf(s, "%d", actor->health);
 	FontOpts fOpts = opts;
 	fOpts.Area = svec2i(width, 11);
 	fOpts.Pad = svec2i(2, 1);
