@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2014-2017, 2019-2022 Cong Xu
+	Copyright (c) 2014-2017, 2019-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -169,6 +169,11 @@ void MapNewLoadCampaignJSON(json_t *root, CampaignSetting *c)
 	// Default one second door close
 	c->DoorOpenTicks = FPS_FRAMELIMIT;
 	LoadInt(&c->DoorOpenTicks, root, "DoorOpenTicks");
+	LoadInt(&c->Lives, root, "Lives");
+	LoadInt(&c->MaxLives, root, "MaxLives");
+	LoadInt(&c->PlayerHP, root, "PlayerHP");
+	LoadInt(&c->PlayerMaxHP, root, "PlayerMaxHP");
+	LoadInt(&c->PlayerExcessHP, root, "PlayerExcessHP");
 }
 
 static void LoadMissionObjectives(
@@ -254,6 +259,22 @@ void LoadMissions(CArray *missions, json_t *missionsNode, int version)
 				}
 			}
 		}
+		json_t *pcsNode =
+			   json_find_first_label(child, "PickupCounts");
+		   if (pcsNode && pcsNode->child)
+		   {
+			   pcsNode = pcsNode->child;
+			   for (json_t *pcNode = pcsNode->child; pcNode;
+					pcNode = pcNode->next)
+			   {
+				   PickupCount pc;
+				   pc.P = StrPickupClass(
+					   json_find_first_label(pcNode, "Pickup")
+						   ->child->text);
+				   LoadInt(&pc.Count, pcNode, "Count");
+				   CArrayPushBack(&m.PickupCounts, &pc);
+			   }
+		   }
 		LoadInt(&m.EnemyDensity, child, "EnemyDensity");
 		LoadWeapons(
 			&m.Weapons, json_find_first_label(child, "Weapons")->child);
