@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2014-2016, 2018-2020, 2022-2023 Cong Xu
+	Copyright (c) 2014-2016, 2018-2020, 2022-2023, 2025 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -28,9 +28,11 @@
 #pragma once
 
 #include "character.h"
+#include "weapon_usage.h"
 
 #define MAX_GUNS 3
 #define MELEE_SLOT 2
+#define GRENADE_SLOT 3
 #define MAX_GRENADES 1
 #define MAX_WEAPONS (MAX_GUNS + MAX_GRENADES)
 // TODO: track accuracy
@@ -47,16 +49,17 @@ typedef struct
 	const WeaponClass *guns[MAX_WEAPONS];
 	CArray ammo; // of int
 	int Lives;
+	int HP;
 
 	NPlayerStats Stats;
 	NPlayerStats Totals;
+	WeaponUsages WeaponUsages;
 
 	// Used for end-of-game score tallying
-	int survived;
 	int hp;
+	int survived;
 	int missions;
 	int lastMission;
-	int allTime, today;
 
 	input_device_e inputDevice;
 	int deviceIndex;
@@ -110,6 +113,7 @@ bool PlayerTrySetUnusedInputDevice(
 int PlayerGetNumWeapons(const PlayerData *p);
 bool PlayerHasGrenadeButton(const PlayerData *p);
 bool PlayerHasWeapon(const PlayerData *p, const WeaponClass *wc);
+bool PlayerHasWeaponUpgrade(const PlayerData *p, const WeaponClass *wc);
 void PlayerAddWeapon(PlayerData *p, const WeaponClass *wc);
 void PlayerAddWeaponToSlot(
 	PlayerData *p, const WeaponClass *wc, const int slot);
@@ -118,4 +122,9 @@ void PlayerAddMinimalWeapons(PlayerData *p);
 bool PlayerUsesAmmo(const PlayerData *p, const int ammoId);
 bool PlayerUsesAnyAmmo(const PlayerData *p);
 int PlayerGetAmmoAmount(const PlayerData *p, const int ammoId);
-void PlayerAddAmmo(PlayerData *p, const int ammoId, const int amount, const bool isFree);
+void PlayerAddAmmo(
+	PlayerData *p, const int ammoId, const int amount, const bool isFree);
+void PlayerSetHP(PlayerData *p, const int hp);
+void PlayerSetLives(PlayerData *p, const int lives);
+
+int CharacterGetStartingHealth(const Character *c, const PlayerData *p);
