@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2013-2022 Cong Xu
+	Copyright (c) 2013-2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -36,7 +36,7 @@
 
 WeaponClasses gWeaponClasses;
 
-static const char *GunTypeStr(const GunType t)
+const char *GunTypeStr(const GunType t)
 {
 	switch (t)
 	{
@@ -49,9 +49,26 @@ static const char *GunTypeStr(const GunType t)
 	}
 }
 
+void GunTypeGetSlotStartEnd(const GunType gt, int *start, int *end)
+{
+	switch (gt)
+	{
+	case GUNTYPE_MELEE:
+		*start = *end = MELEE_SLOT;
+		break;
+	case GUNTYPE_GRENADE:
+		*start = *end = GRENADE_SLOT;
+		break;
+	default:
+		*start = 0;
+		*end = 1;
+		break;
+	}
+}
+
 // Initialise all the static weapon data
 #define VERSION 3
-void WeaponClassesInitialize(WeaponClasses *wcs)
+	void WeaponClassesInitialize(WeaponClasses *wcs)
 {
 	memset(wcs, 0, sizeof *wcs);
 	CArrayInit(&wcs->Guns, sizeof(WeaponClass));
@@ -181,6 +198,14 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		wc->Description = tmp;
 	}
 
+	tmp = NULL;
+	LoadStr(&tmp, node, "Prerequisite");
+	if (tmp != NULL)
+	{
+		CFREE(wc->Prerequisite);
+		wc->Prerequisite = tmp;
+	}
+
 	LoadInt(&wc->Lock, node, "Lock");
 
 	LoadSoundFromNode(&wc->SwitchSound, node, "SwitchSound");
@@ -271,11 +296,10 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 		LoadFloat(&wc->u.Normal.Spread.Width, node, "SpreadWidth");
 		LoadFloat(&wc->u.Normal.AngleOffset, node, "AngleOffset");
 
-		int muzzleHeight = 0;
-		LoadInt(&muzzleHeight, node, "MuzzleHeight");
-		if (muzzleHeight)
+		if (json_find_first_label(node, "MuzzleHeight"))
 		{
-			wc->u.Normal.MuzzleHeight = muzzleHeight * Z_FACTOR;
+			LoadInt(&wc->u.Normal.MuzzleHeight, node, "MuzzleHeight");
+			wc->u.Normal.MuzzleHeight *= Z_FACTOR;
 		}
 		if (json_find_first_label(node, "Elevation"))
 		{
@@ -333,6 +357,8 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 	}
 
 	LoadInt(&wc->Price, node, "Price");
+	wc->OverheatTicks = FPS_FRAMELIMIT;	// default 1 second overheating
+	LoadInt(&wc->OverheatTicks, node, "OverheatTicks");
 
 	LOG(LM_MAP, LL_DEBUG, "loaded %s name(%s) lock(%d)...",
 		GunTypeStr(wc->Type), wc->name, wc->Lock);
@@ -376,7 +402,7 @@ static void LoadWeaponClass(WeaponClass *wc, json_t *node, const int version)
 			wc->u.Normal.Auto ? "true" : "false");
 	}
 
-	LOG(LM_MAP, LL_DEBUG, "price(%d)", wc->Price);
+	LOG(LM_MAP, LL_DEBUG, "price(%d) overheatTicks(%d)", wc->Price, wc->OverheatTicks);
 }
 void WeaponClassesTerminate(WeaponClasses *wcs)
 {
@@ -396,6 +422,7 @@ static void WeaponClassTerminate(WeaponClass *wc)
 {
 	CFREE(wc->name);
 	CFREE(wc->Description);
+	CFREE(wc->Prerequisite);
 	CFREE(wc->DropGun);
 	if (wc->Type == GUNTYPE_MULTI)
 	{
@@ -659,6 +686,42 @@ const BulletClass *WeaponClassGetBullet(
 	CA_FOREACH_END()
 	return NULL;
 }
+const WeaponClass *WeaponClassGetPrerequisite(const WeaponClass *wc)
+{
+	if (wc == NULL || wc->Prerequisite == NULL)
+	{
+		return NULL;
+	}
+	return StrWeaponClass(wc->Prerequisite);
+}
+static const WeaponClass *FindAncestor(const WeaponClass *wc)
+{
+	for (;;)
+	{
+		const WeaponClass *parent = WeaponClassGetPrerequisite(wc);
+		if (parent == NULL)
+		{
+			return wc;
+		}
+		wc = parent;
+	}
+}
+bool WeaponClassesAreRelated(const WeaponClass *wc1, const WeaponClass *wc2)
+{
+	// Find the ancestors of both, and check if they match
+	return wc1 != NULL && wc2 != NULL &&
+		   FindAncestor(wc1) == FindAncestor(wc2);
+}
+
+int WeaponClassFullPrice(const WeaponClass *wc)
+{
+	// Sum of prices for this weapon and all its prerequisites
+	if (wc == NULL)
+	{
+		return 0;
+	}
+	return wc->Price + WeaponClassFullPrice(WeaponClassGetPrerequisite(wc));
+}
 
 void BulletAndWeaponInitialize(
 	BulletClasses *b, WeaponClasses *wcs, const char *bpath, const char *gpath)
