@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2021 Cong Xu
+	Copyright (c) 2013-2021, 2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,7 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define SDL_MAIN_HANDLED
 #include <SDL.h>
 
 #include <cdogs/XGetopt.h>
@@ -224,7 +225,8 @@ static void Display(HandleInputResult result)
 	}
 
 	FontStr(
-		"Press " KMOD_CMD_NAME "+E to edit characters", svec2i(20, h - 20 - FontH() * 2));
+		"Press " KMOD_CMD_NAME "+E to edit characters",
+		svec2i(20, h - 20 - FontH() * 2));
 	FontStr("Press F1 for help", svec2i(20, h - 20 - FontH()));
 
 	UIObjectDraw(
@@ -233,8 +235,7 @@ static void Display(HandleInputResult result)
 
 	if (result.WillDisplayAutomap && mission)
 	{
-		AutomapDraw(
-			gGraphicsDevice.gameWindow.renderer, AUTOMAP_FLAGS_SHOWALL, true);
+		AutomapDraw(&gGraphicsDevice, NULL, AUTOMAP_FLAGS_SHOWALL, true);
 	}
 	else
 	{
@@ -309,6 +310,17 @@ static void DeleteItem(Mission *m, int idx)
 	CArrayDelete(&m->MapObjectDensities, idx);
 }
 
+static void DeletePickup(Mission *m, int idx)
+{
+	if (idx >= (int)m->PickupCounts.size)
+	{
+		// Nothing to delete; do nothing
+		return;
+	}
+	idx = CLAMP(idx, 0, (int)m->PickupCounts.size);
+	CArrayDelete(&m->PickupCounts, idx);
+}
+
 static void AdjustYC(int *yc)
 {
 	Mission *mission = CampaignGetCurrentMission(&gCampaign);
@@ -361,6 +373,13 @@ static void AdjustXC(int yc, int *xc)
 		}
 		break;
 
+	case YC_PICKUPS:
+		if (mission && mission->PickupCounts.size > 0)
+		{
+			*xc = CLAMP_OPPOSITE(*xc, 0, (int)mission->PickupCounts.size - 1);
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -471,7 +490,7 @@ static void Save(void)
 	char dirname[CDOGS_PATH_MAX];
 	PathGetDirname(dirname, lastFile);
 	char buf[CDOGS_PATH_MAX];
-	if (TrySaveFile(buf, &gEventHandlers,dirname, PathGetBasename(lastFile)))
+	if (TrySaveFile(buf, &gEventHandlers, dirname, PathGetBasename(lastFile)))
 	{
 		WindowContextPreRender(&gGraphicsDevice.gameWindow);
 		ClearScreen(&gGraphicsDevice);
@@ -480,7 +499,12 @@ static void Save(void)
 
 		BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.screen);
 		WindowContextPostRender(&gGraphicsDevice.gameWindow);
-		MapArchiveSave(buf, &gCampaign.Setting);
+		if (!MapArchiveSave(buf, &gCampaign.Setting))
+		{
+			SDL_ShowSimpleMessageBox(
+			 SDL_MESSAGEBOX_INFORMATION, "Error", "Error when saving campaign!",
+			 gGraphicsDevice.gameWindow.window);
+		}
 		fileChanged = false;
 		strcpy(lastFile, buf);
 		sAutosaveIndex = 0;
@@ -504,19 +528,20 @@ static void HelpScreen(void)
 		"Common commands\n"
 		"===============\n"
 		"left/right click, page up/down: Increase/decrease value\n"
-		"shift + left/right click:          Increase/decrease number of items\n"
+		"shift + left/right click:          Increase/decrease number of "
+		"items\n"
 		"insert:                         Add new item\n"
 		"delete:                         Delete selected item\n"
 		"arrow keys:                     Move camera\n"
 		"\n"
 		"Other commands\n"
 		"==============\n"
-		"Escape:                         Back or quit\n"
-		KMOD_CMD_NAME "+E:                         Go to character editor\n"
-		KMOD_CMD_NAME"+N:                         New mission\n"
-		KMOD_CMD_NAME"+O:                         Open file\n"
-		KMOD_CMD_NAME"+S:                         Save file\n"
-		KMOD_CMD_NAME"+X, C, V:                   Cut/copy/paste\n"
+		"Escape:                         Back or quit\n" KMOD_CMD_NAME
+		"+E:                         Go to character editor\n" KMOD_CMD_NAME
+		"+N:                         New mission\n" KMOD_CMD_NAME
+		"+O:                         Open file\n" KMOD_CMD_NAME
+		"+S:                         Save file\n" KMOD_CMD_NAME
+		"+X, C, V:                   Cut/copy/paste\n"
 		"tab:                              Preview automap\n"
 		"F1:                               This screen\n";
 	WindowContextPreRender(&gGraphicsDevice.gameWindow);
@@ -555,6 +580,10 @@ static void Delete(int xc, int yc)
 		DeleteItem(mission, xc);
 		break;
 
+	case YC_PICKUPS:
+		DeletePickup(mission, xc);
+		break;
+
 	default:
 		if (yc >= YC_OBJECTIVES)
 		{
@@ -583,7 +612,10 @@ static void Delete(int xc, int yc)
 	Setup(changedMission);
 }
 
-static UIObject *OnUIInput(HandleInputResult *result, const EventHandlers *event, const int m, int *xc, int *yc, int *xcOld, int *ycOld, SDL_Scancode *sc, const Mission *mission);
+static UIObject *OnUIInput(
+	HandleInputResult *result, const EventHandlers *event, const int m,
+	int *xc, int *yc, int *xcOld, int *ycOld, SDL_Scancode *sc,
+	const Mission *mission);
 static void InputInsert(int *xc, const int yc, Mission *mission);
 static void InputDelete(const int xc, const int yc);
 static HandleInputResult HandleInput(
@@ -643,7 +675,8 @@ static HandleInputResult HandleInput(
 	if (m && (m == SDL_BUTTON_LEFT || m == SDL_BUTTON_RIGHT ||
 			  MouseWheel(&gEventHandlers.mouse).y != 0))
 	{
-		o = OnUIInput(&result, &gEventHandlers, m, xc, yc, xcOld, ycOld, &sc, mission);
+		o = OnUIInput(
+			&result, &gEventHandlers, m, xc, yc, xcOld, ycOld, &sc, mission);
 	}
 	if (!brush.IsActive)
 	{
@@ -799,6 +832,9 @@ static HandleInputResult HandleInput(
 		case 'c':
 			MissionTerminate(scrap);
 			MissionCopy(scrap, mission);
+			SDL_ShowSimpleMessageBox(
+				SDL_MESSAGEBOX_INFORMATION, "Editor", "Mission copied",
+				gGraphicsDevice.gameWindow.window);
 			break;
 
 		case 'v':
@@ -904,7 +940,9 @@ static HandleInputResult HandleInput(
 
 		case SDL_SCANCODE_GRAVE:
 			ToggleCollapse(sObjs->Data, 0);
-			OnUIInput(&result, &gEventHandlers, m, xc, yc, xcOld, ycOld, &sc, mission);
+			OnUIInput(
+				&result, &gEventHandlers, m, xc, yc, xcOld, ycOld, &sc,
+				mission);
 			break;
 
 		case SDL_SCANCODE_BACKSPACE:
@@ -924,7 +962,7 @@ static HandleInputResult HandleInput(
 				InputDelete(*xc, *yc);
 			}
 			break;
-		
+
 		case SDL_SCANCODE_TAB:
 			result.WillDisplayAutomap = true;
 			break;
@@ -991,7 +1029,10 @@ static HandleInputResult HandleInput(
 	}
 	return result;
 }
-UIObject *OnUIInput(HandleInputResult *result, const EventHandlers *event, const int m, int *xc, int *yc, int *xcOld, int *ycOld, SDL_Scancode *sc, const Mission *mission)
+UIObject *OnUIInput(
+	HandleInputResult *result, const EventHandlers *event, const int m,
+	int *xc, int *yc, int *xcOld, int *ycOld, SDL_Scancode *sc,
+	const Mission *mission)
 {
 	result->Redraw = true;
 	UIObject *o = NULL;
@@ -1099,6 +1140,15 @@ static void InputInsert(int *xc, const int yc, Mission *mission)
 	}
 	break;
 
+	case YC_PICKUPS: {
+		PickupCount pc;
+		pc.P = PickupClassGetById(&gPickupClasses, 0);
+		pc.Count = 1;
+		CArrayPushBack(&mission->PickupCounts, &pc);
+		*xc = (int)(mission->PickupCounts.size - 1);
+	}
+	break;
+
 	default:
 		if (yc >= YC_OBJECTIVES)
 		{
@@ -1184,9 +1234,10 @@ int main(int argc, char *argv[])
 
 	// Print command line
 	char buf[CDOGS_PATH_MAX];
-	struct option longopts[] = {{"log", required_argument, NULL, 1000},
-								{"logfile", required_argument, NULL, 1001},
-								{0, 0, NULL, 0}};
+	struct option longopts[] = {
+		{"log", required_argument, NULL, 1000},
+		{"logfile", required_argument, NULL, 1001},
+		{0, 0, NULL, 0}};
 	int opt = 0;
 	int idx = 0;
 	while ((opt = getopt_long(argc, argv, "\0:\0", longopts, &idx)) != -1)
@@ -1300,11 +1351,6 @@ int main(int argc, char *argv[])
 		if (!loaded)
 		{
 			RealPath(argv[i], lastFile);
-			if (strchr(lastFile, '.') == NULL &&
-				sizeof lastFile - strlen(lastFile) > 3)
-			{
-				strcat(lastFile, ".cdogscpn");
-			}
 			if (MapNewLoad(lastFile, &gCampaign.Setting) == 0)
 			{
 				loaded = 1;
