@@ -1,29 +1,29 @@
 /*
-    C-Dogs SDL
-    A port of the legendary (and fun) action/arcade cdogs.
-    Copyright (c) 2014, Cong Xu
-    All rights reserved.
+	C-Dogs SDL
+	A port of the legendary (and fun) action/arcade cdogs.
+	Copyright (c) 2014, 2019-2021 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "editor_brush.h"
 
@@ -34,13 +34,12 @@
 #include <cdogs/map_build.h>
 #include <cdogs/mission_convert.h>
 
-
 void EditorBrushInit(EditorBrush *b)
 {
 	memset(b, 0, sizeof *b);
 	b->Type = BRUSHTYPE_POINT;
-	b->MainType = MAP_WALL;
-	b->SecondaryType = MAP_FLOOR;
+	b->MainType = 0;
+	b->SecondaryType = 1;
 	b->PaintType = b->MainType;
 	b->IsActive = 0;
 	b->IsPainting = 0;
@@ -53,7 +52,7 @@ void EditorBrushInit(EditorBrush *b)
 void EditorBrushTerminate(EditorBrush *b)
 {
 	CArrayTerminate(&b->HighlightedTiles);
-	SDL_FreeSurface(b->GuideImageSurface);
+	PicFree(&b->GuideImagePic);
 }
 
 static void EditorBrushHighlightPoint(void *data, struct vec2i p)
@@ -74,8 +73,7 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 	bool useSimpleHighlight = true;
 	switch (b->Type)
 	{
-	case BRUSHTYPE_POINT:	// fallthrough
-	case BRUSHTYPE_ROOM_PAINTER:
+	case BRUSHTYPE_POINT:
 		useSimpleHighlight = true;
 		break;
 	case BRUSHTYPE_LINE:
@@ -90,9 +88,9 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 			BresenhamLineDraw(b->LastPos, b->Pos, &data);
 		}
 		break;
-	case BRUSHTYPE_BOX:	// fallthrough
-	case BRUSHTYPE_ROOM:	// fallthrough
-	case BRUSHTYPE_SET_EXIT:
+	case BRUSHTYPE_BOX:
+	case BRUSHTYPE_BOX_AND_FILL: // fallthrough
+	case BRUSHTYPE_SET_EXIT:	 // fallthrough
 		if (b->IsPainting)
 		{
 			struct vec2i v;
@@ -138,8 +136,8 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 			if (b->IsMoving)
 			{
 				struct vec2i v;
-				struct vec2i offset = svec2i(
-					b->Pos.x - b->DragPos.x, b->Pos.y - b->DragPos.y);
+				struct vec2i offset =
+					svec2i(b->Pos.x - b->DragPos.x, b->Pos.y - b->DragPos.y);
 				useSimpleHighlight = 0;
 				CArrayClear(&b->HighlightedTiles);
 				for (v.y = 0; v.y < b->SelectionSize.y; v.y++)
@@ -182,7 +180,8 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 				{
 					for (v.x = 0; v.x < b->SelectionSize.x; v.x++)
 					{
-						struct vec2i vOffset = svec2i_add(v, b->SelectionStart);
+						struct vec2i vOffset =
+							svec2i_add(v, b->SelectionStart);
 						EditorBrushHighlightPoint(b, vOffset);
 					}
 				}
@@ -200,11 +199,17 @@ void EditorBrushSetHighlightedTiles(EditorBrush *b)
 	}
 }
 
-static void SetTile(Mission *m, struct vec2i pos, unsigned short tile)
+static void SetTile(
+	MapBuilder *mb, Mission *m, const struct vec2i pos, const int tile)
 {
-	if (MissionTrySetTile(m, pos, tile))
+	if (MissionStaticTrySetTile(&m->u.Static, mb->Map->Size, pos, tile))
 	{
-		MapSetTile(&gMap, pos, tile, m);
+		const TileClass *tc = MissionStaticGetTileClass(
+			&mb->mission->u.Static, mb->Map->Size, pos);
+		if (tc != NULL)
+		{
+			MapBuildTile(mb, pos, tc);
+		}
 	}
 }
 
@@ -219,13 +224,16 @@ static void EditorBrushPaintTilesAt(void *data, struct vec2i pos)
 	EditorBrushPaintTilesAtData *paintData = data;
 	EditorBrush *b = paintData->brush;
 	Mission *m = paintData->mission;
+	MapBuilder mb;
+	MapBuilderInit(&mb, &gMap, m, GAME_MODE_NORMAL, NULL);
 	for (v.y = 0; v.y < b->BrushSize; v.y++)
 	{
 		for (v.x = 0; v.x < b->BrushSize; v.x++)
 		{
-			SetTile(m, svec2i_add(pos, v), b->PaintType);
+			SetTile(&mb, m, svec2i_add(pos, v), b->PaintType);
 		}
 	}
+	MapBuilderTerminate(&mb);
 }
 static void EditorBrushPaintLine(EditorBrush *b, Mission *m)
 {
@@ -244,40 +252,15 @@ static void EditorBrushPaintLine(EditorBrush *b, Mission *m)
 	b->IsPainting = 1;
 	b->LastPos = b->Pos;
 }
-// Paint all the edge tiles as a wall, unless they are room tiles already;
-// then paint the interior as room tiles
-static void EditorBrushPaintRoom(EditorBrush *b, Mission *m)
-{
-	struct vec2i v;
-	for (v.y = 0; v.y < b->BrushSize; v.y++)
-	{
-		for (v.x = 0; v.x < b->BrushSize; v.x++)
-		{
-			unsigned short tile = MAP_ROOM;
-			if (v.x == 0 || v.x == b->BrushSize - 1 ||
-				v.y == 0 || v.y == b->BrushSize - 1)
-			{
-				tile = MAP_WALL;
-			}
-			const struct vec2i pos = svec2i_add(b->Pos, v);
-			const unsigned short tileExisting = IMapGet(&gMap, pos);
-			if (tileExisting != MAP_ROOM)
-			{
-				SetTile(m, pos, tile);
-			}
-		}
-	}
-	b->IsPainting = true;
-	b->LastPos = b->Pos;
-}
 typedef struct
 {
+	MapBuilder mb;
 	Mission *m;
-	unsigned short fromType;
-	unsigned short toType;
+	int fromType;
+	int toType;
 } PaintFloodFillData;
-static void MissionFillTile(void *data, struct vec2i v);
-static bool MissionIsTileSame(void *data, struct vec2i v);
+static void MissionFillTile(void *data, const struct vec2i v);
+static bool MissionIsTileSame(void *data, const struct vec2i v);
 EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 {
 	if (!b->IsPainting)
@@ -291,16 +274,13 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 		b->IsPainting = true;
 		EditorBrushPaintLine(b, m);
 		return EDITOR_RESULT_CHANGED;
-	case BRUSHTYPE_LINE:	// fallthrough
-	case BRUSHTYPE_BOX:	// fallthrough
-	case BRUSHTYPE_BOX_FILLED:	// fallthrough
-	case BRUSHTYPE_ROOM:	// fallthrough
+	case BRUSHTYPE_LINE:		 // fallthrough
+	case BRUSHTYPE_BOX:			 // fallthrough
+	case BRUSHTYPE_BOX_FILLED:	 // fallthrough
+	case BRUSHTYPE_BOX_AND_FILL: // fallthrough
 	case BRUSHTYPE_SET_EXIT:
 		// don't paint until the end
 		break;
-	case BRUSHTYPE_ROOM_PAINTER:
-		EditorBrushPaintRoom(b, m);
-		return EDITOR_RESULT_CHANGED;
 	case BRUSHTYPE_SELECT:
 		// Perform state changes if we've started painting
 		if (!b->IsPainting)
@@ -325,26 +305,37 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 		// another type
 		// Don't paint if target already same type
 		// Special case: don't flood-fill doors
-		if ((MissionGetTile(m, b->Pos) & MAP_MASKACCESS) != b->PaintType &&
-			b->PaintType != MAP_DOOR)
+		if (MissionStaticIdTileClass(&m->u.Static, b->PaintType)->Type ==
+			TILE_CLASS_DOOR)
 		{
+			break;
+		}
+		{
+			const int tile =
+				MissionStaticGetTile(&m->u.Static, m->Size, b->Pos);
+			if (tile == b->PaintType)
+			{
+				break;
+			}
 			FloodFillData data;
 			data.Fill = MissionFillTile;
 			data.IsSame = MissionIsTileSame;
 			PaintFloodFillData pData;
+			MapBuilderInit(&pData.mb, &gMap, m, GAME_MODE_NORMAL, NULL);
 			pData.m = m;
-			pData.fromType = MissionGetTile(m, b->Pos) & MAP_MASKACCESS;
+			pData.fromType = tile;
 			pData.toType = b->PaintType;
 			data.data = &pData;
-			if (CFloodFill(b->Pos, &data))
+			const bool flooded = CFloodFill(b->Pos, &data);
+			MapBuilderTerminate(&pData.mb);
+			if (!flooded)
 			{
-				return EDITOR_RESULT_CHANGED;
+				break;
 			}
+			return EDITOR_RESULT_CHANGED;
 		}
-		return EDITOR_RESULT_NONE;
 	case BRUSHTYPE_SET_PLAYER_START:
-		if (MissionGetTile(m, b->Pos) == MAP_ROOM ||
-			MissionGetTile(m, b->Pos) == MAP_FLOOR)
+		if (TileIsClear(MapGetTile(&gMap, b->Pos)))
 		{
 			m->u.Static.Start = b->Pos;
 			return EDITOR_RESULT_CHANGED;
@@ -353,14 +344,19 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_ITEM:
 		if (isMain)
 		{
-			if (MissionStaticTryAddItem(m, b->u.MapObject, b->Pos))
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			const Tile *tileAbove =
+				MapGetTile(&gMap, svec2i(b->Pos.x, b->Pos.y - 1));
+
+			if (MapObjectIsTileOK(b->u.MapObject, tile, tileAbove) &&
+				MissionStaticTryAddItem(&m->u.Static, b->u.MapObject, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
 		else
 		{
-			if (MissionStaticTryRemoveItemAt(m, b->Pos))
+			if (MissionStaticTryRemoveItemAt(&m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -369,14 +365,21 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_CHARACTER:
 		if (isMain)
 		{
-			if (MissionStaticTryAddCharacter(m, b->u.ItemIndex, b->Pos))
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			if (TileIsClear(tile))
+			{
+				CharacterPlace cp = {b->Pos, DIRECTION_DOWN};
+				MissionStaticAddCharacter(&m->u.Static, b->u.ItemIndex, cp);
+				return EDITOR_RESULT_CHANGED_AND_RELOAD;
+			}
+			else if (MissionStaticTryRotateCharacterAt(&m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
 		else
 		{
-			if (MissionStaticTryRemoveCharacterAt(m, b->Pos))
+			if (MissionStaticTryRemoveCharacterAt(&m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -385,15 +388,13 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_OBJECTIVE:
 		if (isMain)
 		{
-			if (MissionStaticTryAddObjective(
-				m, b->u.ItemIndex, b->Index2, b->Pos))
-			{
-				return EDITOR_RESULT_CHANGED_AND_RELOAD;
-			}
+			MissionStaticAddObjective(
+				m, &m->u.Static, b->u.ItemIndex, b->Index2, b->Pos, false);
+			return EDITOR_RESULT_CHANGED_AND_RELOAD;
 		}
 		else
 		{
-			if (MissionStaticTryRemoveObjectiveAt(m, b->Pos))
+			if (MissionStaticTryRemoveObjective(m, &m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
@@ -402,55 +403,81 @@ EditorResult EditorBrushStartPainting(EditorBrush *b, Mission *m, int isMain)
 	case BRUSHTYPE_ADD_KEY:
 		if (isMain)
 		{
-			if (MissionStaticTryAddKey(m, b->u.ItemIndex, b->Pos))
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			if (TileIsClear(tile))
+			{
+				MissionStaticAddKey(&m->u.Static, b->u.ItemIndex, b->Pos);
+				return EDITOR_RESULT_CHANGED_AND_RELOAD;
+			}
+		}
+		else
+		{
+			if (MissionStaticTryRemoveKeyAt(&m->u.Static, b->Pos))
+			{
+				return EDITOR_RESULT_CHANGED_AND_RELOAD;
+			}
+		}
+		break;
+	case BRUSHTYPE_ADD_PICKUP:
+		if (isMain)
+		{
+			const Tile *tile = MapGetTile(&gMap, b->Pos);
+			if (TileCanWalk(tile) &&
+				MissionStaticTryAddPickup(&m->u.Static, b->u.Pickup, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
 		else
 		{
-			if (MissionStaticTryRemoveKeyAt(m, b->Pos))
+			if (MissionStaticTryRemovePickupAt(&m->u.Static, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
 		break;
 	case BRUSHTYPE_SET_KEY:
-		if (isMain || b->u.ItemIndex > 0)
+		if (isMain)
 		{
-			if (MissionStaticTrySetKey(m, b->u.ItemIndex, b->Pos))
+			if (MissionStaticTrySetKey(
+					&m->u.Static, b->u.ItemIndex, m->Size, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
 		else
 		{
-			if (MissionStaticTryUnsetKeyAt(m, b->Pos))
+			if (MissionStaticTryUnsetKeyAt(&m->u.Static, m->Size, b->Pos))
 			{
 				return EDITOR_RESULT_CHANGED_AND_RELOAD;
 			}
 		}
 		break;
 	default:
-		assert(0 && "unknown brush type");
+		CASSERT(false, "unknown brush type");
 		break;
 	}
 	b->IsPainting = 1;
 	return EDITOR_RESULT_NONE;
 }
-static void MissionFillTile(void *data, struct vec2i v)
+static void MissionFillTile(void *data, const struct vec2i v)
 {
 	PaintFloodFillData *pData = data;
-	SetTile(pData->m, v, pData->toType);
+	SetTile(&pData->mb, pData->m, v, pData->toType);
 }
-static bool MissionIsTileSame(void *data, struct vec2i v)
+static bool MissionIsTileSame(void *data, const struct vec2i v)
 {
 	PaintFloodFillData *pData = data;
-	return (MissionGetTile(pData->m, v) & MAP_MASKACCESS) == pData->fromType;
+	if (!Rect2iIsInside(Rect2iNew(svec2i_zero(), pData->mb.Map->Size), v))
+	{
+		return false;
+	}
+	return MissionStaticGetTile(
+			   &pData->mb.mission->u.Static, pData->mb.Map->Size, v) ==
+		   pData->fromType;
 }
 static void EditorBrushPaintBox(
-	EditorBrush *b, Mission *m,
-	unsigned short lineType, unsigned short fillType)
+	EditorBrush *b, Mission *m, const int lineType, const int fillType)
 {
 	// Draw the fill first, then the line
 	// This will create the expected results when brush size is
@@ -458,13 +485,12 @@ static void EditorBrushPaintBox(
 	// aware of brush sizes
 	struct vec2i v;
 	struct vec2i d = svec2i(
-		b->Pos.x > b->LastPos.x ? 1 : -1,
-		b->Pos.y > b->LastPos.y ? 1 : -1);
+		b->Pos.x > b->LastPos.x ? 1 : -1, b->Pos.y > b->LastPos.y ? 1 : -1);
 	EditorBrushPaintTilesAtData paintData;
 	paintData.brush = b;
 	paintData.mission = m;
 	// Draw fill
-	if (fillType != MAP_UNSET)
+	if (fillType >= 0)
 	{
 		b->PaintType = fillType;
 		for (v.y = b->LastPos.y; v.y != b->Pos.y + d.y; v.y += d.y)
@@ -480,7 +506,7 @@ static void EditorBrushPaintBox(
 		}
 	}
 	// Draw edge
-	if (lineType != MAP_UNSET)
+	if (lineType >= 0)
 	{
 		b->PaintType = lineType;
 		for (v.y = b->LastPos.y; v.y != b->Pos.y + d.y; v.y += d.y)
@@ -508,21 +534,17 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 			result = EDITOR_RESULT_CHANGED;
 			break;
 		case BRUSHTYPE_BOX:
-			EditorBrushPaintBox(b, m, b->PaintType, MAP_UNSET);
+			EditorBrushPaintBox(b, m, b->PaintType, -1);
 			result = EDITOR_RESULT_CHANGED;
 			break;
 		case BRUSHTYPE_BOX_FILLED:
 			EditorBrushPaintBox(b, m, b->PaintType, b->PaintType);
 			result = EDITOR_RESULT_CHANGED;
 			break;
-		case BRUSHTYPE_ROOM:
-			EditorBrushPaintBox(b, m, MAP_WALL, MAP_ROOM);
+		case BRUSHTYPE_BOX_AND_FILL:
+			EditorBrushPaintBox(b, m, b->MainType, b->SecondaryType);
 			result = EDITOR_RESULT_CHANGED;
 			break;
-		case BRUSHTYPE_ROOM_PAINTER:
-			// Reload map to update tiles
-			result = EDITOR_RESULT_RELOAD;
-			break;
 		case BRUSHTYPE_SELECT:
 			if (b->IsMoving)
 			{
@@ -530,49 +552,31 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 				// Need to copy all the tiles to a temp buffer first in case
 				// we are moving to an overlapped position
 				CArray movedTiles;
-				struct vec2i v;
-				int i;
 				int delta;
-				CArrayInit(&movedTiles, sizeof(unsigned short));
-				// Copy tiles to temp from selection, setting them to MAP_FLOOR
+				CArrayInit(&movedTiles, m->u.Static.Tiles.elemSize);
+				// Copy tiles to temp from selection, clearing them
 				// in the process
-				for (v.y = 0; v.y < b->SelectionSize.y; v.y++)
-				{
-					for (v.x = 0; v.x < b->SelectionSize.x; v.x++)
-					{
-						struct vec2i vOffset = svec2i_add(v, b->SelectionStart);
-						int idx = vOffset.y * m->Size.x + vOffset.x;
-						unsigned short *tile = CArrayGet(
-							&m->u.Static.Tiles, idx);
-						CArrayPushBack(&movedTiles, tile);
-						*tile = MAP_FLOOR;
-					}
-				}
+				RECT_FOREACH(Rect2iNew(b->SelectionStart, b->SelectionSize))
+				const int tile =
+					MissionStaticGetTile(&m->u.Static, m->Size, _v);
+				CArrayPushBack(&movedTiles, &tile);
+				MissionStaticClearTile(&m->u.Static, m->Size, _v);
+				RECT_FOREACH_END()
 				// Move the selection to the new position
 				b->SelectionStart.x += b->Pos.x - b->DragPos.x;
 				b->SelectionStart.y += b->Pos.y - b->DragPos.y;
 				// Copy tiles to the new area, for parts of the new area that
 				// are valid
-				i = 0;
-				for (v.y = 0; v.y < b->SelectionSize.y; v.y++)
+				RECT_FOREACH(Rect2iNew(b->SelectionStart, b->SelectionSize))
+				if (_v.x >= 0 && _v.x < m->Size.x && _v.y >= 0 &&
+					_v.y < m->Size.y)
 				{
-					for (v.x = 0; v.x < b->SelectionSize.x; v.x++)
-					{
-						struct vec2i vOffset = svec2i_add(v, b->SelectionStart);
-						if (vOffset.x >= 0 && vOffset.x < m->Size.x &&
-							vOffset.y >= 0 && vOffset.y < m->Size.y)
-						{
-							int idx = vOffset.y * m->Size.x + vOffset.x;
-							unsigned short *tileFrom =
-								CArrayGet(&movedTiles, i);
-							unsigned short *tileTo = CArrayGet(
-								&m->u.Static.Tiles, idx);
-							*tileTo = *tileFrom;
-							result = EDITOR_RESULT_CHANGED_AND_RELOAD;
-						}
-						i++;
-					}
+					const int tile = *(int *)CArrayGet(&movedTiles, _i);
+					MissionStaticTrySetTile(&m->u.Static, m->Size, _v, tile);
+					result = EDITOR_RESULT_CHANGED_AND_RELOAD;
 				}
+				RECT_FOREACH_END()
+				CArrayTerminate(&movedTiles);
 				// Update the selection to fit within map boundaries
 				delta = -b->SelectionStart.x;
 				if (delta > 0)
@@ -617,31 +621,34 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 				}
 			}
 			break;
-		case BRUSHTYPE_SET_EXIT:
+		case BRUSHTYPE_SET_EXIT: {
+			struct vec2i exitStart = svec2i_min(b->LastPos, b->Pos);
+			struct vec2i exitEnd = svec2i_max(b->LastPos, b->Pos);
+			// Clamp within map boundaries
+			exitStart = svec2i_clamp(
+				exitStart, svec2i_zero(),
+				svec2i_subtract(m->Size, svec2i_one()));
+			exitEnd = svec2i_clamp(
+				exitEnd, svec2i_zero(),
+				svec2i_subtract(m->Size, svec2i_one()));
+			// Check that size is big enough
+			const struct vec2i size = svec2i_subtract(exitEnd, exitStart);
+			if (size.x < 2 || size.y < 2)
 			{
-				struct vec2i exitStart = svec2i_min(b->LastPos, b->Pos);
-				struct vec2i exitEnd = svec2i_max(b->LastPos, b->Pos);
-				// Clamp within map boundaries
-				exitStart = svec2i_clamp(
-					exitStart, svec2i_zero(), svec2i_subtract(m->Size, svec2i_one()));
-				exitEnd = svec2i_clamp(
-					exitEnd, svec2i_zero(), svec2i_subtract(m->Size, svec2i_one()));
-				// Check that size is big enough
-				struct vec2i size =
-					svec2i_add(svec2i_subtract(exitEnd, exitStart), svec2i_one());
-				if (size.x >= 3 && size.y >= 3)
-				{
-					// Check that exit area has changed
-					if (!svec2i_is_equal(exitStart, m->u.Static.Exit.Start) ||
-						!svec2i_is_equal(exitEnd, m->u.Static.Exit.End))
-					{
-						m->u.Static.Exit.Start = exitStart;
-						m->u.Static.Exit.End = exitEnd;
-						result = EDITOR_RESULT_CHANGED_AND_RELOAD;
-					}
-				}
+				break;
 			}
-			break;
+			// Check that exit area has changed
+			Exit *exit = CArrayGet(&m->u.Static.Exits, b->u.ItemIndex);
+			if (svec2i_is_equal(exitStart, exit->R.Pos) &&
+				svec2i_is_equal(size, exit->R.Size))
+			{
+				break;
+			}
+			exit->R.Pos = exitStart;
+			exit->R.Size = size;
+			result = EDITOR_RESULT_CHANGED_AND_RELOAD;
+		}
+		break;
 		default:
 			// do nothing
 			break;
@@ -651,3 +658,16 @@ EditorResult EditorBrushStopPainting(EditorBrush *b, Mission *m)
 	CArrayClear(&b->HighlightedTiles);
 	return result;
 }
+
+bool EditorBrushTryLoadGuideImage(EditorBrush *b, const char *filename)
+{
+	PicFree(&b->GuideImagePic);
+	SDL_Surface *s = LoadImgToSurface(filename);
+	if (s == NULL)
+		return false;
+	SDL_Surface *sc = SDL_ConvertSurface(s, gGraphicsDevice.Format, 0);
+	SDL_FreeSurface(s);
+	PicLoad(&b->GuideImagePic, svec2i(sc->w, sc->h), svec2i_zero(), sc, false);
+	SDL_FreeSurface(sc);
+	return true;
+}
