@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2013-2016, 2019-2020 Cong Xu
+	Copyright (c) 2013-2016, 2019-2020, 2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -29,8 +29,6 @@
 
 #include <assert.h>
 
-#include <SDL_image.h>
-
 #include <cdogs/draw/draw.h>
 #include <cdogs/draw/draw_actor.h>
 #include <cdogs/events.h>
@@ -132,7 +130,8 @@ static void DrawCharacter(
 	UNUSED(g);
 	EditorBrushAndCampaign *data = vData;
 	CharacterStore *store = &data->Campaign->Setting.characters;
-	const Character *c = CArrayGet(&store->OtherChars, data->Brush.u.ItemIndex);
+	const Character *c =
+		CArrayGet(&store->OtherChars, data->Brush.u.ItemIndex);
 	DrawCharacterSimple(
 		c,
 		svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2)),
@@ -167,16 +166,26 @@ static void DrawObjective(
 		DrawCharacterSimple(c, pos, DIRECTION_DOWN, false, false, c->Gun);
 	}
 	break;
-	case OBJECTIVE_COLLECT: {
-		const Pic *p = CPicGetPic(&obj->u.Pickup->Pic, 0);
-		pos = svec2i_subtract(pos, svec2i_scale_divide(p->size, 2));
+	case OBJECTIVE_COLLECT:
+		CA_FOREACH(const PickupClass *, pc, obj->u.Pickups)
+		const Pic *p = CPicGetPic(&(*pc)->Pic, 0);
+		const struct vec2i ppos =
+			svec2i_subtract(pos, svec2i_scale_divide(p->size, 2));
+		PicRender(
+			p, g->gameWindow.renderer, ppos, colorWhite, 0, svec2_one(),
+			SDL_FLIP_NONE, Rect2iZero());
+		pos = svec2i_add(pos, svec2i(4, 2));
+		CA_FOREACH_END()
+		break;
+	case OBJECTIVE_DESTROY:
+		CA_FOREACH(const MapObject *, mo, obj->u.MapObjects)
+		struct vec2i offset;
+		const Pic *p = MapObjectGetPic(*mo, &offset);
 		PicRender(
 			p, g->gameWindow.renderer, pos, colorWhite, 0, svec2_one(),
 			SDL_FLIP_NONE, Rect2iZero());
-	}
-	break;
-	case OBJECTIVE_DESTROY:
-		DisplayMapItem(pos, obj->u.MapObject);
+		pos = svec2i_add(pos, svec2i(4, 2));
+		CA_FOREACH_END()
 		break;
 	default:
 		assert(0 && "invalid objective type");
@@ -300,7 +309,7 @@ UIObject *CreateAddItemObjs(struct vec2i pos, EditorBrush *brush, Campaign *co)
 
 static void DrawMapItem(
 	UIObject *o, GraphicsDevice *g, struct vec2i pos, void *vData);
-static char *MakeMapObjectTooltip(const MapObject *mo);
+static char *MakeMapObjectTooltipSimple(const MapObject *mo);
 static bool AddMapItemBrushObjFunc(UIObject *o, MapObject *mo, void *vData)
 {
 	if (mo->Type == MAP_OBJECT_TYPE_PICKUP_SPAWNER)
@@ -313,7 +322,7 @@ static bool AddMapItemBrushObjFunc(UIObject *o, MapObject *mo, void *vData)
 	o->OnUnfocusFunc = DeactivateIndexedEditorBrush;
 	((IndexedEditorBrush *)o->Data)->Brush = vData;
 	((IndexedEditorBrush *)o->Data)->u.MapObject = mo;
-	o->Tooltip = MakeMapObjectTooltip(mo);
+	o->Tooltip = MakeMapObjectTooltipSimple(mo);
 	return true;
 }
 static void DrawMapItem(
@@ -325,9 +334,9 @@ static void DrawMapItem(
 		svec2i_add(svec2i_add(pos, o->Pos), svec2i_scale_divide(o->Size, 2)),
 		brush->u.MapObject);
 }
-static char *MakeMapObjectTooltip(const MapObject *mo)
+static char *MakeMapObjectTooltipSimple(const MapObject *mo)
 {
-	// Add a descriptive tooltip for the map object
+	// Add a descriptive tooltip for the map object, without placement flags
 	char buf[512];
 	// Construct text representing explosion guns
 	char exBuf[256];
@@ -351,7 +360,7 @@ static char *MakeMapObjectTooltip(const MapObject *mo)
 	return tmp;
 }
 
-static char *MakePickupTooltip(const MapObject *mo);
+static char *MakePickupSpawnerTooltip(const MapObject *mo);
 static bool AddPickupSpawnerBrushObjFunc(
 	UIObject *o, MapObject *mo, void *vData)
 {
@@ -365,10 +374,10 @@ static bool AddPickupSpawnerBrushObjFunc(
 	o->OnUnfocusFunc = DeactivateIndexedEditorBrush;
 	((IndexedEditorBrush *)o->Data)->Brush = vData;
 	((IndexedEditorBrush *)o->Data)->u.MapObject = mo;
-	o->Tooltip = MakePickupTooltip(mo);
+	o->Tooltip = MakePickupSpawnerTooltip(mo);
 	return true;
 }
-static char *MakePickupTooltip(const MapObject *mo)
+static char *MakePickupSpawnerTooltip(const MapObject *mo)
 {
 	char *tmp;
 	CASSERT(mo->u.PickupClass->Name != NULL, "No pickup name");
