@@ -40,6 +40,7 @@
 
 #define NO_GUN_LABEL "(None)"
 #define AMMO_LABEL "Ammo..."
+#define UTIL_LABEL "Utilities..."
 #define END_MENU_LABEL "(End)"
 #define EQUIP_MENU_WIDTH 80
 #define EQUIP_MENU_SLOT_HEIGHT 40
@@ -48,17 +49,11 @@
 #define SLOT_BORDER 3
 #define AMMO_LEVEL_W 2
 
-static GunType SlotType(const int slot)
+static bool InSlot(const WeaponClass *wc, const int slot)
 {
-	if (slot < MELEE_SLOT)
-	{
-		return GUNTYPE_NORMAL;
-	}
-	else if (slot == MELEE_SLOT)
-	{
-		return GUNTYPE_MELEE;
-	}
-	return GUNTYPE_GRENADE;
+	int slotStart, slotEnd;
+	GunTypeGetSlotStartEnd(wc->Type, &slotStart, &slotEnd);
+	return slot >= slotStart && slot <= slotEnd;
 }
 
 static int CountNumGuns(const EquipMenu *data, const int slot)
@@ -70,7 +65,7 @@ static int CountNumGuns(const EquipMenu *data, const int slot)
 	// Count total guns
 	int numGuns = 0;
 	CA_FOREACH(const WeaponClass *, wc, data->weapons)
-	if ((*wc)->Type != SlotType(slot))
+	if (!InSlot(*wc, slot))
 	{
 		continue;
 	}
@@ -99,7 +94,10 @@ static bool IsSlotDisabled(const EquipMenu *data, const int slot)
 	{
 		return !PlayerUsesAnyAmmo(pData);
 	}
-	// TODO: util menus
+	if (slot == data->utilSlot)
+	{
+		return false;
+	}
 	return false;
 }
 
@@ -122,12 +120,11 @@ static void DrawEquipSlot(
 		}
 		else
 		{
-			const color_t bg = {0, 255, 255, 64};
 			// Add 1px padding
 			const struct vec2i bgPos = svec2i_subtract(pos, svec2i_one());
 			const struct vec2i bgSize =
 				svec2i(EQUIP_MENU_WIDTH / 2 + 2, h + 2);
-			DrawRectangle(g, bgPos, bgSize, bg, true);
+			DrawRectangle(g, bgPos, bgSize, colorSelectedBG, true);
 
 			color = colorRed;
 		}
@@ -173,10 +170,30 @@ static void DrawEquipSlot(
 		g, pData, pData->guns[slot], mask, svec2i(bgPos.x, bgPos.y + FontH()),
 		svec2i(slotSize.x, slotSize.y - 2 * FontH() - 1));
 
-	if (data->SlotHasNew[slot])
+	const Pic *arrow = PicManagerGetPic(&gPicManager, "hud/arrow");
+	const struct vec2i metaPos =
+		svec2i(pos.x + EQUIP_MENU_WIDTH / 2 - 6, y + 13);
+	const struct vec2i arrowPos = svec2i(metaPos.x - 4, metaPos.y);
+	switch (data->SlotMeta[slot])
 	{
-		DrawCross(
-			g, svec2i(pos.x + EQUIP_MENU_WIDTH / 2 - 6, y + 13), colorGreen);
+	case META_NONE:
+		break;
+	case META_NEW:
+		DrawCross(g, metaPos, colorGreen);
+		break;
+	case META_UPGRADE:
+		PicRender(
+			arrow, g->gameWindow.renderer, arrowPos, colorGreen, 0,
+			svec2_one(), SDL_FLIP_NONE, Rect2iZero());
+		break;
+	case META_DOWNGRADE:
+		PicRender(
+			arrow, g->gameWindow.renderer, arrowPos, colorRed, 0, svec2_one(),
+			SDL_FLIP_VERTICAL, Rect2iZero());
+		break;
+	default:
+		CASSERT(false, "unexpected");
+		break;
 	}
 
 	// Draw price
@@ -229,6 +246,10 @@ static void DrawEquipMenu(
 			{
 				costDiff = AmmoMenuSelectedCostDiff(&d->ammoMenu);
 			}
+			else if (d->slot == d->utilSlot)
+			{
+				costDiff = UtilMenuSelectedCostDiff(&d->utilMenu);
+			}
 			if (costDiff != 0)
 			{
 				// Draw price diff
@@ -262,6 +283,17 @@ static void DrawEquipMenu(
 			AMMO_LABEL, selected, disabled, colorWhite);
 		y += UTIL_MENU_SLOT_HEIGHT;
 	}
+	if (d->utilSlot >= 0)
+	{
+		const bool selected = d->slot == d->utilSlot;
+		DisplayMenuItem(
+			g,
+			Rect2iNew(
+				svec2i(CENTER_X(pos, size, FontStrSize(UTIL_LABEL).x), y),
+				FontStrSize(UTIL_LABEL)),
+			UTIL_LABEL, selected, false, colorWhite);
+		y += UTIL_MENU_SLOT_HEIGHT;
+	}
 
 	y += 8;
 	const WeaponClass *gun = NULL;
@@ -288,12 +320,12 @@ static int HandleInputEquipMenu(int cmd, void *data)
 	PlayerData *p = PlayerDataGetByUID(d->PlayerUID);
 
 	int newSlot = d->slot;
-	if (cmd & CMD_BUTTON1)
+	if (Button1(cmd))
 	{
 		MenuPlaySound(MENU_SOUND_ENTER);
 		return 1;
 	}
-	else if (cmd & CMD_BUTTON2)
+	else if (Button2(cmd))
 	{
 		if (d->slot < MAX_WEAPONS)
 		{
@@ -302,7 +334,7 @@ static int HandleInputEquipMenu(int cmd, void *data)
 			AnimatedCounterReset(&d->Cash, p->Totals.Score);
 		}
 	}
-	else if (cmd & CMD_LEFT)
+	else if (Left(cmd))
 	{
 		if (d->slot < MAX_WEAPONS && (d->slot & 1) == 1)
 		{
@@ -321,7 +353,7 @@ static int HandleInputEquipMenu(int cmd, void *data)
 			}
 		}
 	}
-	else if (cmd & CMD_RIGHT)
+	else if (Right(cmd))
 	{
 		if (d->slot < MAX_WEAPONS && (d->slot & 1) == 0)
 		{
@@ -340,7 +372,7 @@ static int HandleInputEquipMenu(int cmd, void *data)
 			}
 		}
 	}
-	else if (cmd & CMD_UP)
+	else if (Up(cmd))
 	{
 		if (d->slot >= MAX_WEAPONS)
 		{
@@ -371,7 +403,7 @@ static int HandleInputEquipMenu(int cmd, void *data)
 			}
 		}
 	}
-	else if (cmd & CMD_DOWN)
+	else if (Down(cmd))
 	{
 		if (d->slot < d->endSlot)
 		{
@@ -424,6 +456,54 @@ static int HandleInputEquipMenu(int cmd, void *data)
 }
 
 static bool HasWeapon(const CArray *weapons, const WeaponClass *wc);
+static void ResetSlotMeta(EquipMenu *menu)
+{
+	const PlayerData *pData = PlayerDataGetByUID(menu->PlayerUID);
+
+	// Update SlotMeta
+	for (int i = 0; i < MAX_WEAPONS; i++)
+	{
+		menu->SlotMeta[i] = META_NONE;
+		const WeaponClass *slotWC = pData->guns[i];
+		CA_FOREACH(const WeaponClass *, wc, menu->weapons)
+		if (!InSlot(*wc, i))
+		{
+			continue;
+		}
+		const bool isNew =
+			*(const bool *)CArrayGet(&menu->weaponIsNew, _ca_index);
+		DrawGunMeta meta = isNew ? META_NEW : META_NONE;
+		if (slotWC != NULL)
+		{
+			// Check if this slot has an upgrade/downgrade
+			if (slotWC == WeaponClassGetPrerequisite(*wc))
+			{
+				meta = META_UPGRADE;
+			}
+			else if (WeaponClassGetPrerequisite(slotWC) == *wc)
+			{
+				meta = META_DOWNGRADE;
+			}
+		}
+		menu->SlotMeta[i] = MAX(menu->SlotMeta[i], meta);
+		CA_FOREACH_END()
+	}
+}
+static bool HasWeapon(const CArray *weapons, const WeaponClass *wc)
+{
+	if (weapons == NULL)
+	{
+		return true;
+	}
+	CA_FOREACH(const WeaponClass *, wc2, *weapons)
+	if (*wc2 == wc)
+	{
+		return true;
+	}
+	CA_FOREACH_END()
+	return false;
+}
+
 void EquipMenuCreate(
 	EquipMenu *menu, const CArray *weapons, const CArray *prevWeapons,
 	const int numPlayers, const int player, const int playerUID,
@@ -447,25 +527,8 @@ void EquipMenuCreate(
 	CA_FOREACH(const WeaponClass *, wc, menu->weapons)
 	const bool isNew = !HasWeapon(prevWeapons, *wc);
 	CArrayPushBack(&menu->weaponIsNew, &isNew);
-	if (isNew)
-	{
-		if ((*wc)->Type == GUNTYPE_GRENADE)
-		{
-			menu->SlotHasNew[MAX_GUNS] = true;
-		}
-		else if ((*wc)->Type == GUNTYPE_MELEE)
-		{
-			menu->SlotHasNew[MELEE_SLOT] = true;
-		}
-		else
-		{
-			for (int i = 0; i < MELEE_SLOT; i++)
-			{
-				menu->SlotHasNew[i] = true;
-			}
-		}
-	}
 	CA_FOREACH_END()
+	ResetSlotMeta(menu);
 
 	switch (numPlayers)
 	{
@@ -494,6 +557,7 @@ void EquipMenuCreate(
 	// Check how many util menu items there are
 	menu->endSlot = MAX_WEAPONS;
 	menu->ammoSlot = -1;
+	menu->utilSlot = -1;
 	if (gCampaign.Setting.BuyAndSell)
 	{
 		if (gCampaign.Setting.Ammo)
@@ -510,6 +574,9 @@ void EquipMenuCreate(
 				}
 			}
 		}
+		// Can always buy lives; buying HP depends on game settings
+		menu->utilSlot = menu->endSlot;
+		menu->endSlot++;
 	}
 
 	// Create equipped weapons menu
@@ -559,7 +626,7 @@ void EquipMenuCreate(
 	// For AI players, pre-pick their weapons and go straight to menu end
 	if (pData->inputDevice == INPUT_DEVICE_AI)
 	{
-		menu->slot = menu->endSlot;
+		menu->ms.current = NULL;
 		menu->equipping = false;
 		AICoopSelectWeapons(pData, player, weapons);
 	}
@@ -573,22 +640,10 @@ void EquipMenuCreate(
 		}
 		AmmoMenuCreate(
 			&menu->ammoMenu, playerUID, pos, menu->size, handlers, graphics);
+		UtilMenuCreate(
+			&menu->utilMenu, playerUID, pos, menu->size, handlers, graphics);
 	}
 }
-static bool HasWeapon(const CArray *weapons, const WeaponClass *wc)
-{
-	if (weapons == NULL)
-	{
-		return true;
-	}
-	CA_FOREACH(const WeaponClass *, wc2, *weapons)
-	if (*wc2 == wc)
-	{
-		return true;
-	}
-	CA_FOREACH_END()
-	return false;
-}
 
 void EquipMenuTerminate(EquipMenu *menu)
 {
@@ -597,6 +652,7 @@ void EquipMenuTerminate(EquipMenu *menu)
 		WeaponMenuTerminate(&menu->weaponMenus[i]);
 	}
 	AmmoMenuTerminate(&menu->ammoMenu);
+	UtilMenuTerminate(&menu->utilMenu);
 	MenuSystemTerminate(&menu->ms);
 	CArrayTerminate(&menu->weapons);
 	CArrayTerminate(&menu->weaponIsNew);
@@ -611,10 +667,15 @@ void EquipMenuUpdate(EquipMenu *menu, const int cmd)
 	{
 		WeaponMenuUpdate(&menu->weaponMenus[menu->slot], cmd);
 		menu->equipping = menu->weaponMenus[menu->slot].Active;
-		// If weapons changed, reset ammo menu
+		// If weapons changed, reset menus
 		if (!menu->equipping)
 		{
+			for (int i = 0; i < MAX_WEAPONS; i++)
+			{
+				WeaponMenuReset(&menu->weaponMenus[i]);
+			}
 			AmmoMenuReset(&menu->ammoMenu);
+			ResetSlotMeta(menu);
 		}
 	}
 	else if (menu->slot == menu->ammoSlot && menu->ammoMenu.Active)
@@ -625,6 +686,14 @@ void EquipMenuUpdate(EquipMenu *menu, const int cmd)
 		}
 		menu->equipping = menu->ammoMenu.Active;
 	}
+	else if (menu->slot == menu->utilSlot && menu->utilMenu.Active)
+	{
+		if (UtilMenuUpdate(&menu->utilMenu, cmd))
+		{
+			AnimatedCounterReset(&menu->Cash, p->Totals.Score);
+		}
+		menu->equipping = menu->utilMenu.Active;
+	}
 	else
 	{
 		MenuProcessCmd(&menu->ms, cmd);
@@ -642,6 +711,10 @@ void EquipMenuUpdate(EquipMenu *menu, const int cmd)
 				{
 					AmmoMenuActivate(&menu->ammoMenu);
 				}
+				else if (menu->slot == menu->utilSlot)
+				{
+					UtilMenuActivate(&menu->utilMenu);
+				}
 				else
 				{
 					CASSERT(false, "unknown menu slot");
@@ -676,4 +749,8 @@ void EquipMenuDraw(const EquipMenu *menu)
 	{
 		AmmoMenuDraw(&menu->ammoMenu);
 	}
+	else if (menu->slot == menu->utilSlot)
+	{
+		UtilMenuDraw(&menu->utilMenu);
+	}
 }
