@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2022 Cong Xu
+	Copyright (c) 2013-2022, 2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -69,14 +69,13 @@
 #include <cdogs/pickup.h>
 
 #include "briefing_screens.h"
-#include "hiscores.h"
 #include "loading_screens.h"
 #include "prep.h"
 #include "screens_end.h"
 
 static void PlayerSpecialCommands(TActor *actor, const int cmd)
 {
-	if ((cmd & CMD_BUTTON2) && CMD_HAS_DIRECTION(cmd))
+	if (Button2(cmd) && CMD_HAS_DIRECTION(cmd))
 	{
 		if (ConfigGetEnum(&gConfig, "Game.SwitchMoveStyle") ==
 				SWITCHMOVE_SLIDE &&
@@ -86,8 +85,8 @@ static void PlayerSpecialCommands(TActor *actor, const int cmd)
 		}
 	}
 	else if (
-		!(actor->lastCmd & CMD_BUTTON2) && (cmd & CMD_BUTTON2) &&
-		!actor->specialCmdDir && !actor->CanPickupSpecial &&
+		!Button2(actor->lastCmd) && Button2(cmd) && !actor->specialCmdDir &&
+		!actor->CanPickupSpecial &&
 		!(ConfigGetEnum(&gConfig, "Game.SwitchMoveStyle") ==
 			  SWITCHMOVE_SLIDE &&
 		  CMD_HAS_DIRECTION(cmd)))
@@ -103,9 +102,7 @@ static void PlayerSpecialCommands(TActor *actor, const int cmd)
 		}
 		else
 		{
-			const PlayerData *p = PlayerDataGetByUID(actor->PlayerUID);
-			const bool allGuns = p == NULL || !PlayerHasGrenadeButton(p);
-			ActorTrySwitchWeapon(actor, allGuns);
+			ActorTrySwitchWeapon(actor);
 		}
 	}
 }
@@ -136,6 +133,17 @@ static void RunGameReset(RunGameData *rData)
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.bkg);
 	CameraReset(&rData->Camera);
 }
+static void OnGfxChangeCallback(void *data, const bool resetBg)
+{
+	RunGameData *rData = data;
+	if (resetBg)
+	{
+		RunGameReset(rData);
+	}
+	rData->pm.ms.size = svec2i(
+		rData->pm.ms.graphics->cachedConfig.Res.x,
+		rData->pm.ms.graphics->cachedConfig.Res.y);
+}
 static void RunGameTerminate(GameLoopData *data)
 {
 	RunGameData *rData = data->Data;
@@ -218,6 +226,10 @@ static void RunGameOnEnter(GameLoopData *data)
 		}
 	}
 
+	PauseMenuInit(
+		&rData->pm, &gEventHandlers, &gGraphicsDevice, OnGfxChangeCallback,
+		rData);
+
 	rData->m->state = MISSION_STATE_WAITING;
 	rData->m->isDone = false;
 	rData->m->DoneCounter = 0;
@@ -266,6 +278,7 @@ static void RunGameOnExit(GameLoopData *data)
 	CA_FOREACH_END()
 	CArrayTerminate(&rData->ammoSpawners);
 	CameraTerminate(&rData->Camera);
+	PauseMenuTerminate(&rData->pm);
 
 	// Draw background
 	GrafxRedrawBackground(&gGraphicsDevice, rData->Camera.lastPosition);
@@ -313,17 +326,14 @@ static void RunGameInput(GameLoopData *data)
 	}
 	memset(rData->cmds, 0, sizeof rData->cmds);
 	int cmdAll = 0;
-	int idx = 0;
-	input_device_e pausingDevice = INPUT_DEVICE_UNSET;
-	input_device_e firstPausingDevice = INPUT_DEVICE_UNSET;
 	if (GetNumPlayers(PLAYER_ANY, false, true) == 0)
 	{
 		// If no players, allow default keyboard to control camera
 		rData->cmds[0] = GetKeyboardCmd(&gEventHandlers.keyboard, 0, false);
-		firstPausingDevice = INPUT_DEVICE_KEYBOARD;
 	}
 	else
 	{
+		int idx = 0;
 		for (int i = 0; i < (int)gPlayerDatas.size; i++, idx++)
 		{
 			const PlayerData *p = CArrayGet(&gPlayerDatas, i);
@@ -332,87 +342,25 @@ static void RunGameInput(GameLoopData *data)
 				idx--;
 				continue;
 			}
-			if (firstPausingDevice == INPUT_DEVICE_UNSET)
-			{
-				firstPausingDevice = p->inputDevice;
-			}
 			rData->cmds[idx] = GetGameCmd(&gEventHandlers, p);
 			cmdAll |= rData->cmds[idx];
-
-			// Only allow the first player to escape
-			// Use keypress otherwise the player will quit immediately
-			if (idx == 0 && (rData->cmds[idx] & CMD_ESC) &&
-				!(rData->lastCmds[idx] & CMD_ESC))
-			{
-				pausingDevice = p->inputDevice;
-			}
 		}
 	}
-	if (KeyIsPressed(&gEventHandlers.keyboard, SDL_SCANCODE_ESCAPE))
-	{
-		pausingDevice = INPUT_DEVICE_KEYBOARD;
-	}
-
-	// Check if any controllers are unplugged
-	rData->controllerUnplugged = false;
-	CA_FOREACH(const PlayerData, p, gPlayerDatas)
-	if (p->inputDevice == INPUT_DEVICE_UNSET && p->IsLocal)
-	{
-		rData->controllerUnplugged = true;
-		break;
-	}
-	CA_FOREACH_END()
 
 	// If in Superhot(tm) Mode, don't update unless there was an input in this
 	// or the last frame
 	data->SkipNextFrame = data->SuperhotMode && !lastCmdAll;
 
-	// Check if:
-	// - escape was pressed, or
-	// - window lost focus
-	// - controller unplugged
-	// If the game is paused, unpause if a button is released
-	// If the game was not paused, enter pause mode
-	// If the game was paused and escape was pressed, exit the game
-	if (rData->pausingDevice != INPUT_DEVICE_UNSET && AnyButton(lastCmdAll) &&
-		!AnyButton(cmdAll))
+	const bool wasPaused = PauseMenuIsShown(&rData->pm);
+
+	// Update and check if we want to quit
+	if (PauseMenuUpdate(&rData->pm, rData->cmds, rData->lastCmds))
 	{
-		rData->pausingDevice = INPUT_DEVICE_UNSET;
-	}
-	else if (rData->controllerUnplugged || gEventHandlers.HasLostFocus)
-	{
-		// Pause the game
-		rData->pausingDevice = firstPausingDevice;
-		rData->isMap = false;
-		SoundPlay(&gSoundDevice, StrSound("menu_error"));
-	}
-	else if (pausingDevice != INPUT_DEVICE_UNSET)
-	{
-		if (rData->pausingDevice != INPUT_DEVICE_UNSET)
-		{
-			// Already paused; exit
-			GameEvent e = GameEventNew(GAME_EVENT_MISSION_END);
-			e.u.MissionEnd.IsQuit = true;
-			GameEventsEnqueue(&gGameEvents, e);
-			// Need to unpause to process the quit
-			rData->pausingDevice = INPUT_DEVICE_UNSET;
-			rData->controllerUnplugged = false;
-			// Don't skip exiting the game
-			data->SkipNextFrame = false;
-			SoundPlay(&gSoundDevice, StrSound("menu_back"));
-		}
-		else
-		{
-			// Pause the game
-			rData->pausingDevice = pausingDevice;
-			rData->isMap = false;
-			SoundPlay(&gSoundDevice, StrSound("menu_back"));
-		}
+		// Need to unpause to process the quit
+		data->SkipNextFrame = true;
 	}
 
-	const bool paused = rData->pausingDevice != INPUT_DEVICE_UNSET ||
-						rData->controllerUnplugged;
-	if (!paused)
+	if (!PauseMenuIsShown(&rData->pm))
 	{
 		// Check if automap key is pressed by any player
 		// Toggle
@@ -427,6 +375,23 @@ static void RunGameInput(GameLoopData *data)
 				&gSoundDevice,
 				StrSound(rData->isMap ? "map_open" : "map_close"));
 		}
+		if (wasPaused)
+		{
+			// Clear all user inputs if we're using the pause menu
+			memset(rData->cmds, 0, sizeof rData->cmds);
+			memset(rData->lastCmds, 0, sizeof rData->lastCmds);
+			KeyLockKeys(&gEventHandlers.keyboard);
+			JoyLock(&gEventHandlers.joysticks);
+		}
+	}
+
+	if (!wasPaused && PauseMenuIsShown(&rData->pm))
+	{
+		// Don't show map if pause menu is shown
+		rData->isMap = false;
+		// Clear all user inputs if we're using the pause menu
+		memset(rData->cmds, 0, sizeof rData->cmds);
+		memset(rData->lastCmds, 0, sizeof rData->lastCmds);
 	}
 
 	CameraInput(&rData->Camera, rData->cmds[0], rData->lastCmds[0]);
@@ -466,8 +431,7 @@ static GameLoopResult RunGameUpdate(GameLoopData *data, LoopRunner *l)
 	// If we're not hosting a net game,
 	// don't update if the game has paused or has automap shown
 	// Important: don't consider paused if we are trying to quit
-	const bool paused = rData->pausingDevice != INPUT_DEVICE_UNSET ||
-						rData->controllerUnplugged || rData->isMap;
+	const bool paused = PauseMenuIsShown(&rData->pm) || rData->isMap;
 	if (!gCampaign.IsClient && !ConfigGetBool(&gConfig, "StartServer") &&
 		paused && !gEventHandlers.HasQuit)
 	{
@@ -559,7 +523,8 @@ static GameLoopResult RunGameUpdate(GameLoopData *data, LoopRunner *l)
 				rData->cmds[idx] = AICoopGetCmd(player, ticksPerFrame);
 			}
 			PlayerSpecialCommands(player, rData->cmds[idx]);
-			CommandActor(player, rData->cmds[idx], ticksPerFrame);
+			rData->cmds[idx] =
+				CommandActor(player, rData->cmds[idx], ticksPerFrame);
 		}
 	}
 
@@ -603,10 +568,6 @@ static void NextLoop(RunGameData *rData, LoopRunner *l)
 			break;
 		}
 	}
-	else
-	{
-		nextScreen = HighScoresScreen(rData->co, &gGraphicsDevice);
-	}
 	LoopRunnerPush(l, ScreenLoading("Debriefing...", true, nextScreen, true));
 	if (!HasRounds(rData->co->Entry.Mode) && !rData->co->IsComplete)
 	{
@@ -618,11 +579,7 @@ static void PersistPlayerWeaponsAndAmmo(PlayerData *p)
 	if (!IsPlayerAlive(p))
 		return;
 	const TActor *a = ActorGetByUID(p->ActorUID);
-	for (int i = 0; i < MAX_WEAPONS; i++)
-	{
-		p->guns[i] = a->guns[i].Gun;
-	}
-	CArrayCopy(&p->ammo, &a->ammo);
+	ActorPersistPlayerWeaponsAndAmmo(a);
 }
 static void CheckMissionCompletion(const struct MissionOptions *mo)
 {
@@ -707,14 +664,13 @@ static void RunGameDraw(GameLoopData *data)
 	BlitClearBuf(&gGraphicsDevice);
 	CameraDrawMode(&rData->Camera);
 	HUDDraw(
-		&rData->Camera.HUD, rData->pausingDevice, rData->controllerUnplugged,
-		rData->Camera.NumViews);
+		&rData->Camera.HUD, rData->Camera.NumViews,
+		PauseMenuIsShown(&rData->pm));
+	PauseMenuDraw(&rData->pm);
 	// Draw automap if enabled
 	if (rData->isMap)
 	{
-		AutomapDraw(
-			gGraphicsDevice.gameWindow.renderer, 0,
-			rData->Camera.HUD.showExit);
+		AutomapDraw(&gGraphicsDevice, NULL, 0, rData->Camera.HUD.showExit);
 	}
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.hud);
 
@@ -724,7 +680,7 @@ static void RunGameDraw(GameLoopData *data)
 		if (IsAutoMapEnabled(gCampaign.Entry.Mode))
 		{
 			AutomapDraw(
-				gGraphicsDevice.secondWindow.renderer, 0,
+				&gGraphicsDevice, gGraphicsDevice.secondWindow.renderer, 0,
 				rData->Camera.HUD.showExit);
 		}
 		BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.hud2);
