@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2014, 2016-2021 Cong Xu
+	Copyright (c) 2013-2014, 2016-2021, 2025 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -259,6 +259,84 @@ static Rect2i MenuGetSubmenuBounds(const MenuSystem *ms, const int idx)
 	return Rect2iNew(pos, SubmenuGetSize(ms, menu, idx));
 }
 
+static void MoveIndexToNextEnabledSubmenu(menu_t *menu, const bool isDown)
+{
+	if (menu->u.normal.index >= (int)menu->u.normal.subMenus.size)
+	{
+		menu->u.normal.index = (int)menu->u.normal.subMenus.size - 1;
+	}
+	const int firstIndex = menu->u.normal.index;
+	bool isFirst = true;
+	// Move the selection to the next non-disabled submenu
+	for (;;)
+	{
+		const menu_t *currentSubmenu =
+			CArrayGet(&menu->u.normal.subMenus, menu->u.normal.index);
+		if (!currentSubmenu->isDisabled)
+		{
+			break;
+		}
+		if (menu->u.normal.index == firstIndex && !isFirst)
+		{
+			break;
+		}
+		isFirst = false;
+		if (isDown)
+		{
+			menu->u.normal.index++;
+			if (menu->u.normal.index == (int)menu->u.normal.subMenus.size)
+			{
+				menu->u.normal.index = 0;
+			}
+		}
+		else
+		{
+			menu->u.normal.index--;
+			if (menu->u.normal.index == -1)
+			{
+				menu->u.normal.index = (int)menu->u.normal.subMenus.size - 1;
+			}
+		}
+	}
+}
+// Constrain the menu scroll so that the selected item is always in view
+// and we don't scroll the menu off the screen
+static void MenuClampScroll(menu_t *menu)
+{
+	const int nMenuItems = MenuGetNumMenuItemsShown(menu);
+	menu->u.normal.scroll = CLAMP(
+		menu->u.normal.scroll, MAX(0, menu->u.normal.index - nMenuItems + 1),
+		MIN((int)menu->u.normal.subMenus.size - nMenuItems,
+			menu->u.normal.index));
+	if (menu->u.normal.index < menu->u.normal.scroll)
+	{
+		menu->u.normal.scroll = menu->u.normal.index;
+	}
+}
+static void MenuChangeIndex(menu_t *menu, const int d)
+{
+	if (d < 0)
+	{
+		menu->u.normal.index--;
+		if (menu->u.normal.index == -1)
+		{
+			menu->u.normal.index = (int)menu->u.normal.subMenus.size - 1;
+		}
+		MoveIndexToNextEnabledSubmenu(menu, false);
+		MenuPlaySound(MENU_SOUND_SWITCH);
+	}
+	else if (d > 0)
+	{
+		menu->u.normal.index++;
+		if (menu->u.normal.index == (int)menu->u.normal.subMenus.size)
+		{
+			menu->u.normal.index = 0;
+		}
+		MoveIndexToNextEnabledSubmenu(menu, true);
+		MenuPlaySound(MENU_SOUND_SWITCH);
+	}
+}
+
 static GameLoopResult DefaultMenuUpdate(GameLoopData *data, LoopRunner *l);
 static void DefaultMenuDraw(GameLoopData *data);
 GameLoopData *MenuLoop(MenuSystem *menu)
@@ -277,35 +355,51 @@ static void DefaultMenuDraw(GameLoopData *data)
 }
 void MenuUpdateMouse(MenuSystem *ms)
 {
-	if (!MouseHasMoved(&ms->handlers->mouse))
-	{
-		return;
-	}
 	menu_t *menu = ms->current;
 	if (menu == NULL || !MenuTypeHasSubMenus(menu->type))
 	{
 		return;
 	}
-	// Get mouse position and change menu
-	menu->mouseHover = false;
-	for (int i = 0; i < (int)menu->u.normal.subMenus.size; i++)
+	if (MouseHasMoved(&ms->handlers->mouse))
 	{
-		const Rect2i bounds = MenuGetSubmenuBounds(ms, i);
-		if (!Rect2iIsInside(bounds, ms->handlers->mouse.currentPos))
+		// Get mouse position and change menu
+		menu->mouseHover = false;
+		for (int i = 0; i < (int)menu->u.normal.subMenus.size; i++)
 		{
-			continue;
-		}
-		menu->mouseHover = true;
-		if (menu->u.normal.index != i)
-		{
-			const menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, i);
-			if (!subMenu->isDisabled)
+			const Rect2i bounds = MenuGetSubmenuBounds(ms, i);
+			if (!Rect2iIsInside(bounds, ms->handlers->mouse.currentPos))
 			{
-				menu->u.normal.index = i;
-				MenuPlaySound(MENU_SOUND_SWITCH);
+				continue;
 			}
+			menu->mouseHover = true;
+			if (menu->u.normal.index != i)
+			{
+				const menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, i);
+				if (!subMenu->isDisabled)
+				{
+					menu->u.normal.index = i;
+					MenuPlaySound(MENU_SOUND_SWITCH);
+				}
+			}
+			break;
+		}
+	}
+	// Scroll menu with mouse wheel
+	const int dWheel = MouseWheel(&ms->handlers->mouse).y;
+	if (dWheel != 0)
+	{
+		const int dd = dWheel > 0 ? 1 : -1;
+		for (int i = 0; i != dWheel; i += dd)
+		{
+			const int origScroll = menu->u.normal.scroll;
+			menu->u.normal.scroll += dd;
+			MenuClampScroll(menu);
+			if (menu->u.normal.scroll == origScroll)
+			{
+				break;
+			}
+			MenuChangeIndex(menu, dd);
 		}
-		break;
 	}
 }
 GameLoopResult MenuUpdate(MenuSystem *ms)
@@ -363,47 +457,6 @@ void MenuReset(MenuSystem *menu)
 	menu->current = menu->root;
 }
 
-static void MoveIndexToNextEnabledSubmenu(menu_t *menu, const bool isDown)
-{
-	if (menu->u.normal.index >= (int)menu->u.normal.subMenus.size)
-	{
-		menu->u.normal.index = (int)menu->u.normal.subMenus.size - 1;
-	}
-	const int firstIndex = menu->u.normal.index;
-	bool isFirst = true;
-	// Move the selection to the next non-disabled submenu
-	for (;;)
-	{
-		const menu_t *currentSubmenu =
-			CArrayGet(&menu->u.normal.subMenus, menu->u.normal.index);
-		if (!currentSubmenu->isDisabled)
-		{
-			break;
-		}
-		if (menu->u.normal.index == firstIndex && !isFirst)
-		{
-			break;
-		}
-		isFirst = false;
-		if (isDown)
-		{
-			menu->u.normal.index++;
-			if (menu->u.normal.index == (int)menu->u.normal.subMenus.size)
-			{
-				menu->u.normal.index = 0;
-			}
-		}
-		else
-		{
-			menu->u.normal.index--;
-			if (menu->u.normal.index == -1)
-			{
-				menu->u.normal.index = (int)menu->u.normal.subMenus.size - 1;
-			}
-		}
-	}
-}
-
 void MenuDisableSubmenu(menu_t *menu, int idx)
 {
 	menu_t *subMenu = CArrayGet(&menu->u.normal.subMenus, idx);
@@ -425,11 +478,10 @@ void MenuSetDisabled(menu_t *menu, const bool isDisabled)
 
 menu_t *MenuGetSubmenuByName(menu_t *menu, const char *name)
 {
-	CA_FOREACH(menu_t, subMenu, menu->u.normal.subMenus)
-	if (strcmp(subMenu->name, name) == 0)
-	{
-		return subMenu;
-	}
+	CASSERT(menu->type == MENU_TYPE_NORMAL, "invalid menu type");
+	CA_FOREACH(menu_t, submenu, menu->u.normal.subMenus)
+	if (strcmp(submenu->name, name) == 0)
+		return submenu;
 	CA_FOREACH_END()
 	return NULL;
 }
@@ -469,11 +521,10 @@ struct vec2i DisplayMenuItem(
 	}
 	if (selected)
 	{
-		const color_t bg = {0, 255, 255, 64};
 		// Add 1px padding
 		const struct vec2i bgPos = svec2i_subtract(bounds.Pos, svec2i_one());
 		const struct vec2i bgSize = svec2i_add(bounds.Size, svec2i(2, 2));
-		DrawRectangle(g, bgPos, bgSize, bg, true);
+		DrawRectangle(g, bgPos, bgSize, colorSelectedBG, true);
 		return FontStrMask(s, bounds.Pos, colorRed);
 	}
 	if (!ColorEquals(color, colorTransparent))
@@ -969,7 +1020,7 @@ static int MenuOptionGetIntValue(const menu_t *menu)
 // returns menu to change to, NULL if no change
 menu_t *MenuProcessEscCmd(menu_t *menu);
 menu_t *MenuProcessButtonCmd(MenuSystem *ms, menu_t *menu, int cmd);
-void MenuChangeIndex(menu_t *menu, int cmd);
+static void MenuChangeIndexCmd(menu_t *menu, const int cmd);
 
 void MenuProcessCmd(MenuSystem *ms, int cmd)
 {
@@ -979,8 +1030,8 @@ void MenuProcessCmd(MenuSystem *ms, int cmd)
 	{
 		goto bail;
 	}
-	if (cmd == CMD_ESC || (cmd & CMD_BUTTON2) ||
-		((cmd & CMD_LEFT) && menu->u.normal.isSubmenusAlt))
+	if (cmd == CMD_ESC || Button2(cmd) ||
+		(Left(cmd) && menu->u.normal.isSubmenusAlt))
 	{
 		menuToChange = MenuProcessEscCmd(menu);
 		if (menuToChange != NULL)
@@ -992,7 +1043,8 @@ void MenuProcessCmd(MenuSystem *ms, int cmd)
 	}
 	if (menu->type == MENU_TYPE_CUSTOM)
 	{
-		if (menu->u.customData.inputFunc(cmd, menu->u.customData.data))
+		if (menu->u.customData.inputFunc == NULL ||
+			menu->u.customData.inputFunc(cmd, menu->u.customData.data))
 		{
 			ms->current = menu->parentMenu;
 			goto bail;
@@ -1007,7 +1059,7 @@ void MenuProcessCmd(MenuSystem *ms, int cmd)
 			ms->current = menuToChange;
 			goto bail;
 		}
-		MenuChangeIndex(menu, cmd);
+		MenuChangeIndexCmd(menu, cmd);
 	}
 
 bail:
@@ -1066,26 +1118,25 @@ menu_t *MenuProcessButtonCmd(MenuSystem *ms, menu_t *menu, int cmd)
 		case MENU_TYPE_NORMAL:
 		case MENU_TYPE_OPTIONS:
 		case MENU_TYPE_CUSTOM:
-			if (subMenu->u.normal.isSubmenusAlt ? (cmd & CMD_RIGHT)
-												: (cmd & CMD_BUTTON1))
+			if (subMenu->u.normal.isSubmenusAlt ? Right(cmd) : (Button1(cmd)))
 			{
 				return subMenu;
 			}
 			break;
 		case MENU_TYPE_BACK:
-			if (cmd & CMD_BUTTON1)
+			if (Button1(cmd))
 			{
 				return menu->parentMenu;
 			}
 			break;
 		case MENU_TYPE_QUIT:
-			if (cmd & CMD_BUTTON1)
+			if (Button1(cmd))
 			{
 				return subMenu; // caller will check if subMenu type is QUIT
 			}
 			break;
 		case MENU_TYPE_RETURN:
-			if (cmd & CMD_BUTTON1)
+			if (Button1(cmd))
 			{
 				return subMenu;
 			}
@@ -1188,7 +1239,7 @@ static void ChangeKey(
 	MenuPlaySound(MENU_SOUND_ENTER);
 }
 
-void MenuChangeIndex(menu_t *menu, int cmd)
+static void MenuChangeIndexCmd(menu_t *menu, const int cmd)
 {
 	// Ignore if no submenus
 	if (menu->u.normal.subMenus.size == 0)
@@ -1198,33 +1249,13 @@ void MenuChangeIndex(menu_t *menu, int cmd)
 
 	if (Up(cmd))
 	{
-		menu->u.normal.index--;
-		if (menu->u.normal.index == -1)
-		{
-			menu->u.normal.index = (int)menu->u.normal.subMenus.size - 1;
-		}
-		MoveIndexToNextEnabledSubmenu(menu, false);
-		MenuPlaySound(MENU_SOUND_SWITCH);
+		MenuChangeIndex(menu, -1);
 	}
 	else if (Down(cmd))
 	{
-		menu->u.normal.index++;
-		if (menu->u.normal.index == (int)menu->u.normal.subMenus.size)
-		{
-			menu->u.normal.index = 0;
-		}
-		MoveIndexToNextEnabledSubmenu(menu, true);
-		MenuPlaySound(MENU_SOUND_SWITCH);
-	}
-	const int nMenuItems = MenuGetNumMenuItemsShown(menu);
-	menu->u.normal.scroll = CLAMP(
-		menu->u.normal.scroll, MAX(0, menu->u.normal.index - nMenuItems + 1),
-		MIN((int)menu->u.normal.subMenus.size - 1,
-			menu->u.normal.index + nMenuItems - 1));
-	if (menu->u.normal.index < menu->u.normal.scroll)
-	{
-		menu->u.normal.scroll = menu->u.normal.index;
+		MenuChangeIndex(menu, 1);
 	}
+	MenuClampScroll(menu);
 }
 
 void MenuActivate(MenuSystem *ms, menu_t *menu, int cmd)
