@@ -1,9 +1,8 @@
+#define SDL_MAIN_HANDLED 1
 #include <cbehave/cbehave.h>
 
 #include <c_array.h>
 
-#include <SDL_joystick.h>
-
 #include <utils.h>
 
 // Stubs
